Imports Microsoft.VisualBasic
Imports System
Imports System.Collections.Generic
Imports System.Diagnostics
Imports System.IO.IsolatedStorage
Imports System.Linq
Imports System.Net
Imports System.Net.NetworkInformation
Imports System.Text
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Input
Imports System.Windows.Media
Imports System.Windows.Media.Effects
Imports System.Windows.Shapes
Imports System.Windows.Threading
Imports System.Xml.Linq

Namespace SilverTwitMSDN
    Partial Public Class Page
        Inherits UserControl
#Region "Fields"

        Private ReadOnly domain As String = "localhost:6124" 'TODO: Domain for the SilverTwitMSDNWS
        Private ReadOnly userName As String = "USERNAME" ' TODO: Twitter UserName should be entered into the SilverTwit app through a configuration screen.
        Private ReadOnly password As String = "PASSWORD" ' TODO: Twitter Password should be entered into the SilverTwit app through a configuration screen.
        Private tweetType_Renamed As TweetType = TweetType.All

        Private Const TWEET_STORAGE As String = "SilverTwitTweetStorage"
        Private stats As New OOBStats()
        Private timer As DispatcherTimer

#End Region

#Region "Enums"

        Private Enum TweetType
            Replies
            All
            DirectMessages
        End Enum

#End Region

#Region "Properties"

        Public ReadOnly Property BaseUri() As String
            Get
                Return String.Format("http://{0}/TwitterRESTService.svc", domain)
            End Get
        End Property

#End Region

#Region "Constructors"

        Public Sub New()
            InitializeComponent()
            AddHandler Me.Loaded, AddressOf Page_Loaded
            AddHandler PostTweetButton.Click, AddressOf PostTweetButton_Click
            AddHandler DirectMessageButton.Click, AddressOf DirectMessageButton_Click
            AddHandler ReplyButton.Click, AddressOf ReplyButton_Click
            AddHandler ShowRepliesButton.Click, AddressOf ShowRepliesButton_Click
            AddHandler ShowDirectMessagesButton.Click, AddressOf ShowDirectMessagesButton_Click
            AddHandler ShowTimelineButton.Click, AddressOf ShowTimelineButton_Click
            AddHandler TweetTextBox.KeyDown, AddressOf TweetTextBox_KeyDown
            AddHandler TweetTextBox.TextChanged, AddressOf TweetTextBox_TextChanged

            AddHandler InstallButton.Click, AddressOf InstallButton_Click
            AddHandler NetworkChange.NetworkAddressChanged, AddressOf NetworkChange_NetworkAddressChanged
            AddHandler Application.Current.InstallStateChanged, AddressOf Network_InstallStateChanged

            SetOutOfBrowserStats()
            StartTimer()
            Me.DataContext = stats
        End Sub


#End Region

#Region "Out of Browser Events"

        Private Sub NetworkChange_NetworkAddressChanged(ByVal sender As Object, ByVal e As EventArgs)
            stats.IsNetworkAvailable = NetworkInterface.GetIsNetworkAvailable()

            If NetworkInterface.GetIsNetworkAvailable() Then
                ' get the stored tweet, try to post it
                If IsolatedStorageSettings.ApplicationSettings.Contains(TWEET_STORAGE) Then
                    PostTweet(IsolatedStorageSettings.ApplicationSettings(TWEET_STORAGE).ToString())
                End If
            End If
        End Sub

        Private Sub InstallButton_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
            Try
                Application.Current.Install() ' take the app OOB
            Catch opex As InvalidOperationException
                MessageBox.Show("Application is already Installed to the desktop.")
            Catch ex As Exception
                MessageBox.Show("Application could not be Installed to the desktop." & Environment.NewLine & ex.Message)
            End Try
        End Sub

        Private Sub Network_InstallStateChanged(ByVal sender As Object, ByVal e As EventArgs)
            SetNetworkInstallState()
        End Sub

#End Region

#Region "Out of Browser Helpers"

        Private Sub SetOutOfBrowserStats()
            SetNetworkInstallState()
            stats.IsNetworkAvailable = NetworkInterface.GetIsNetworkAvailable()
            SetOfflineStatus()
        End Sub

        Private Sub SetOfflineStatus()
            If Application.Current.IsRunningOutOfBrowser Then
                stats.RunningModeMessage = "Running Out of Browser"
            Else
                stats.RunningModeMessage = "Running In Browser"
            End If
        End Sub

        Private Function CheckNetworkStatus() As Boolean
            If NetworkInterface.GetIsNetworkAvailable() Then
                Return True
            End If

            MessageBox.Show("A network connection cannot be detected.")
            Return False
        End Function

        Private Sub SetNetworkInstallState()
            Dim installState As InstallState = Application.Current.InstallState
            Select Case installState
                Case installState.NotInstalled
                    stats.InstallStateMessage = "Running Online"
                Case installState.Installing
                    stats.InstallStateMessage = "Installing"
                Case installState.Installed
                    stats.InstallStateMessage = "Installed"
                Case installState.InstallFailed
                    stats.InstallStateMessage = "Install Failed"
                Case Else
                    Throw New ArgumentOutOfRangeException()
            End Select
        End Sub

#End Region

#Region "Event Handlers"

        Private Sub timer_Tick(ByVal sender As Object, ByVal e As EventArgs)
            RefreshTweets()
        End Sub

        Private Sub Page_Loaded(ByVal sender As Object, ByVal e As RoutedEventArgs)
            tweetType_Renamed = TweetType.All
            RefreshTweets()
        End Sub

        Private Sub PostTweetButton_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
            'if (!CheckNetworkStatus()) return;

            PostTweet()
        End Sub

        Private Sub ShowRepliesButton_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
            If (Not CheckNetworkStatus()) Then
                Return
            End If

            tweetType_Renamed = TweetType.Replies
            RefreshTweets()
        End Sub

        Private Sub ShowDirectMessagesButton_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
            If (Not CheckNetworkStatus()) Then
                Return
            End If

            tweetType_Renamed = TweetType.DirectMessages
            RefreshTweets()
        End Sub

        Private Sub ShowTimelineButton_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
            If (Not CheckNetworkStatus()) Then
                Return
            End If

            tweetType_Renamed = TweetType.All
            RefreshTweets()
        End Sub

        Private Sub ReplyButton_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
            InsertTinyButtonItem("@")
        End Sub

        Private Sub DirectMessageButton_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
            InsertTinyButtonItem("d")
        End Sub

        Private Sub TweetTextBox_KeyDown(ByVal sender As Object, ByVal e As KeyEventArgs)
            If e.Key = Key.Enter Then
                PostTweet()
                PostTweetButton.Focus()
            End If
        End Sub

        Private Sub TweetTextBox_TextChanged(ByVal sender As Object, ByVal e As TextChangedEventArgs)
            If TweetTextBox.Text.Length > 140 Then
                TweetTextBox.Text = TweetTextBox.Text.Substring(0, 140)
            End If
        End Sub

#End Region

#Region "Helper Methods"

        Private Sub StartTimer()
            timer = New DispatcherTimer()
            timer.Interval = New TimeSpan(0, 3, 0)
            AddHandler timer.Tick, AddressOf timer_Tick
            timer.Start()
        End Sub

        Private Sub InsertTinyButtonItem(ByVal symbol As String)
            Dim tweet As TweetStatus = TryCast(TweetsListBox.SelectedItem, TweetStatus)
            If tweet Is Nothing Then
                Return
            End If

            TweetTextBox.Focus()

            If TweetTextBox.Text.Length = 0 Then
                TweetTextBox.Text = String.Format("{0}{1}", symbol, tweet.User.ScreenName)
            Else
                TweetTextBox.Text += String.Format(" {0}{1}", symbol, tweet.User.ScreenName)
            End If
        End Sub

        Private Sub LoadDirectMessages(ByVal xmlElement As XElement)
            Dim ns As XNamespace = ""
            Dim twitterQuery = _
             From msg In xmlElement.Descendants(ns + "direct_message") _
             Let sender = msg.Element(ns + "sender") _
             Select New TweetStatus With { _
                .TwitterId = (msg.Element(ns.ToString() & "id").Value).ToLong(), _
                .CreatedAt = (msg.Element(ns.ToString() & "created_at").Value).ToDateTime(), _
                .Text = msg.Element(ns.ToString() & "text").Value, _
                .User = New User With { _
                     .UserId = (sender.Element(ns.ToString() & "id").Value).ToLong(), _
                       .Name = sender.Element(ns.ToString() & "name").Value, _
                       .ScreenName = sender.Element(ns.ToString() & "screen_name").Value, _
                       .Description = sender.Element(ns.ToString() & "description").Value, _
                       .Location = sender.Element(ns.ToString() & "location").Value, _
                       .ProfileImageUrl = sender.Element(ns.ToString() & "profile_image_url").Value, _
                       .Url = sender.Element(ns.ToString() & "url").Value, _
                       .Protected = (sender.Element(ns.ToString() & "protected").Value).ToBool(), _
                       .FollowersCount = (sender.Element(ns.ToString() & "followers_count").Value).ToLong()}}
            
            Dim statusList As List(Of TweetStatus) = twitterQuery.ToList()
            TweetsListBox.DataContext = statusList
        End Sub

        Private Sub LoadTweets(ByVal xmlElement As XElement)
            Dim ns As XNamespace = ""
            Dim twitterQuery = _
             From status In xmlElement.Descendants(ns + "status") _
             Let user = status.Element(ns + "user") _
             Select New TweetStatus With { _
                .TwitterId = (status.Element(ns.ToString() & "id").Value).ToLong(), _
                .CreatedAt = (status.Element(ns.ToString() & "created_at").Value).ToDateTime(), _
                .Text = status.Element(ns.ToString() & "text").Value, _
                .Source = status.Element(ns.ToString() & "source").Value, _
                .Truncated = Convert.ToBoolean(status.Element(ns.ToString() & "truncated").Value), _
                .InReplyToStatusId = status.Element(ns.ToString() & "in_reply_to_status_id").Value, _
                .InReplyToUserId = status.Element(ns.ToString() & "in_reply_to_user_id").Value, _
                .Favorited = (status.Element(ns.ToString() & "favorited").Value).ToBool(), _
                .User = New User With { _
                    .UserId = (user.Element(ns.ToString() & "id").Value).ToLong(), _
                    .Name = user.Element(ns.ToString() & "name").Value, _
                    .ScreenName = user.Element(ns.ToString() & "screen_name").Value, _
                    .Description = user.Element(ns.ToString() & "description").Value, _
                    .Location = user.Element(ns.ToString() & "location").Value, _
                    .ProfileImageUrl = user.Element(ns.ToString() & "profile_image_url").Value, _
                    .Url = user.Element(ns.ToString() & "url").Value, _
                    .Protected = (user.Element(ns.ToString() & "protected").Value).ToBool(), _
                    .FollowersCount = (user.Element(ns.ToString() & "followers_count").Value).ToLong()}}


            Dim statusList As List(Of TweetStatus) = twitterQuery.ToList()
            TweetsListBox.DataContext = statusList
        End Sub

#End Region

#Region "Asynchronous Invocations"

        Private Sub RefreshTweets()
            TweetsListBox.Opacity = 0.5
            TweetsListBox.Effect = New BlurEffect With {.Radius = 3}

            Dim tweetTypeText As String = String.Empty
            Select Case tweetType_Renamed
                Case TweetType.All
                    tweetTypeText = "all"
                Case TweetType.DirectMessages
                    tweetTypeText = "dm"
                Case TweetType.Replies
                    tweetTypeText = "replies"
            End Select

            Dim uriString As String = String.Format("{0}/Tweet/{1}/{2}/?pw={3}", BaseUri, userName, tweetTypeText, password)
            Dim uri As New Uri(uriString)

            Dim svc As New WebClient()
            AddHandler svc.DownloadStringCompleted, AddressOf SvcDownloadStringCompleted
            svc.DownloadStringAsync(uri)
        End Sub

        Private Sub PostTweet(ByVal tweetText As String)
            Dim uriString As String = String.Format("{0}/Tweet/{1}/?pw={2}", BaseUri, userName, password)
            Dim uri As New Uri(uriString)
            Dim svc As New WebClient()
            svc.Headers("Content-type") = "application/xml"
            svc.Encoding = Encoding.UTF8
            Dim xml As String = String.Format("<TweetStatus><Text>{0}</Text></TweetStatus>", tweetText)
            AddHandler svc.UploadStringCompleted, AddressOf SvcUploadStringCompleted
            svc.UploadStringAsync(uri, "POST", xml)
        End Sub

        Private Sub PostTweet()
            Dim tweetText As String = TweetTextBox.Text
            If (Not NetworkInterface.GetIsNetworkAvailable()) Then
                If IsolatedStorageSettings.ApplicationSettings.Contains(TWEET_STORAGE) Then
                    MessageBox.Show("Network connection is not available and only 1 tweet can be stored for delayed posting to Twitter.")
                Else
                    IsolatedStorageSettings.ApplicationSettings.Add(TWEET_STORAGE, tweetText)
                    MessageBox.Show("Network connection is not available. The post will be stored until connectivity returns.")
                End If
                Return
            End If

            PostTweet(tweetText)
        End Sub

#End Region

#Region "Asynchronous Completion Handlers"

        Private Sub SvcDownloadStringCompleted(ByVal sender As Object, ByVal e As DownloadStringCompletedEventArgs)
            Try
                Dim xmlElement As XElement = XElement.Parse(e.Result)
                If xmlElement.Descendants("direct_message").Any() Then
                    LoadDirectMessages(xmlElement)
                Else
                    LoadTweets(xmlElement)
                End If
            Catch ex As Exception
                ' TODO: log the exception
                'MessageBox.Show(ex.Message);
                Console.WriteLine(ex.Message)
            Finally
                TweetsListBox.Effect = Nothing
                TweetsListBox.Opacity = 1
            End Try
        End Sub

        Private Sub SvcUploadStringCompleted(ByVal sender As Object, ByVal e As UploadStringCompletedEventArgs)
            If e.Error Is Nothing Then
                RefreshTweets()
                If IsolatedStorageSettings.ApplicationSettings.Contains(TWEET_STORAGE) Then
                    IsolatedStorageSettings.ApplicationSettings.Remove(TWEET_STORAGE)
                End If
                TweetTextBox.Text = ""
            Else
                'TODO: Have fun with it
                'MessageBox.Show("Could not send your post to Twitter, will try again in a few :-)" + Environment.NewLine + e.Error.Message);
            End If
        End Sub

#End Region
    End Class
End Namespace
