Imports Microsoft.VisualBasic
Imports System
Imports System.IO
Imports System.Linq
Imports System.Net
Imports System.ServiceModel.Activation
Imports System.ServiceModel.Web
Imports System.Text
Imports System.Web
Imports System.Xml.Linq

Namespace SilverTwitMSDNWS
	<AspNetCompatibilityRequirements(RequirementsMode := AspNetCompatibilityRequirementsMode.Allowed)> _
	Public Class TwitterRESTService
		Implements ITwitterRESTService
		#Region "Constants"

		Private Const TWEET_SOURCE As String = "SilverTwit"

		#End Region

		Public Sub New()
			ServicePointManager.Expect100Continue = False
		End Sub

		#Region "Public Web Methods"

		Public Function FindTweets(ByVal username As String, ByVal tweetType As String, ByVal password As String) As XElement Implements ITwitterRESTService.FindTweets
			Dim outResponse As OutgoingWebResponseContext = WebOperationContext.Current.OutgoingResponse

			If String.IsNullOrEmpty(username) OrElse String.IsNullOrEmpty(password) OrElse String.IsNullOrEmpty(tweetType) Then
				Dim ctx As WebOperationContext = WebOperationContext.Current
				ctx.OutgoingResponse.StatusCode = HttpStatusCode.BadRequest
				Return Nothing
			End If

			Dim uriString As String = String.Empty
			Select Case tweetType
				Case "all"
					uriString = String.Format("http://twitter.com/statuses/friends_timeline/{0}.xml?count=50", username)
				Case "dm"
					uriString = String.Format("http://twitter.com/direct_messages.xml")
				Case "replies"
					uriString = String.Format("http://twitter.com/statuses/replies.xml")
				Case Else
					outResponse.StatusCode = HttpStatusCode.BadRequest
					Return Nothing
			End Select
			outResponse.StatusCode = HttpStatusCode.OK
			Return GetRequest(uriString, username, password, outResponse)
		End Function

		Public Function FindPublicTweets() As XElement Implements ITwitterRESTService.FindPublicTweets
			Dim outResponse As OutgoingWebResponseContext = WebOperationContext.Current.OutgoingResponse
			Return GetRequest("http://twitter.com/statuses/public_timeline.xml", outResponse)
		End Function

		Public Sub AddTweet(ByVal username As String, ByVal password As String, ByVal newTweetStatus As TweetStatus) Implements ITwitterRESTService.AddTweet
			'System.Net.ServicePointManager.Expect100Continue = false;
			Dim uri As String = "http://twitter.com/statuses/update.xml"
			Dim inRequest As IncomingWebRequestContext = WebOperationContext.Current.IncomingRequest
			Dim outResponse As OutgoingWebResponseContext = WebOperationContext.Current.OutgoingResponse
			'WebOperationContext ctx = WebOperationContext.Current;
			Dim webRequest As WebRequest = HttpWebRequest.Create(uri)

			If newTweetStatus.Text Is Nothing Then
				outResponse.StatusCode = HttpStatusCode.BadRequest
				Return
			End If

			Dim message As String = newTweetStatus.Text
			message = message.Trim()
			If message.Length > 140 Then
				outResponse.StatusCode = HttpStatusCode.BadRequest
				Return
			End If

			' The TWEET_SOURCE constant below is a class level constant, set to "SilverTwit".
			' It defines the name of the source program talking to Twitter.
			Dim parameters As String = String.Format("status={0}&source={1}", HttpUtility.HtmlEncode(message), TWEET_SOURCE)

			If String.IsNullOrEmpty(username) OrElse String.IsNullOrEmpty(password) Then
				outResponse.StatusCode = HttpStatusCode.BadRequest
				Return
			End If

			' The username and password variables below are set as fields of this class.
			webRequest.Credentials = New NetworkCredential(username, password)
			webRequest.ContentType = "application/x-www-form-urlencoded"
			webRequest.Method = "POST"
			Dim encoding As New ASCIIEncoding()
			Dim bytes() As Byte = encoding.GetBytes(parameters)

			Dim os As Stream = Nothing
			Try
				webRequest.ContentLength = bytes.Length
				os = webRequest.GetRequestStream()
				os.Write(bytes, 0, bytes.Length)
			Catch ex As WebException
				' Not needed really, since 500 is the default status code for an exception
				outResponse.StatusCode = HttpStatusCode.InternalServerError
				Return
			Finally
				If os IsNot Nothing Then
					os.Close()
				End If
			End Try

			Try
				Dim webResponse As WebResponse = webRequest.GetResponse()
				'if (webResponse == null)
				'{
				'    return;
				'}

				'StreamReader sr = new StreamReader(webResponse.GetResponseStream());
				'returnData = sr.ReadToEnd().Trim();
				'sr.Close();
			Catch ex As WebException
				outResponse.StatusCode = HttpStatusCode.InternalServerError
				Return
			End Try

			' Set the outgoing status and response headers
			'outResponse.StatusCode = HttpStatusCode.Created;
			Dim template As New UriTemplate("Tweet/{username}/?pw={password}")
			Dim newUri As Uri = template.BindByPosition(inRequest.UriTemplateMatch.BaseUri, username, password)
			outResponse.SetStatusAsCreated(newUri)
			Return
		End Sub

		#End Region

		#Region "Private Helper Methods"

		Private Function GetRequest(ByVal uri As String, ByVal outResponse As OutgoingWebResponseContext) As XElement
			Return GetRequest(uri, String.Empty, String.Empty, outResponse)
		End Function

		Private Function GetRequest(ByVal uri As String, ByVal username As String, ByVal password As String, ByVal outResponse As OutgoingWebResponseContext) As XElement
			Try
				SetNoCache(outResponse)

				Dim wc As New WebClient()
				If username.Length > 0 AndAlso password.Length > 0 Then
					wc.Credentials = New NetworkCredential(username, password)
				End If
				Dim data As String = wc.DownloadString(uri)
				Dim xdoc As XDocument = XDocument.Parse(data)

				Dim xmlElement As XElement
				If xdoc.Descendants("direct-messages").Any() Then
					xmlElement = xdoc.Element("direct-messages")
				Else
					xmlElement = xdoc.Element("statuses")
				End If

				outResponse.StatusCode = HttpStatusCode.OK
				Return xmlElement
			Catch ex As Exception
				'TODO: Do something better :) ... log the error too
				' Eat the error for now
				Console.WriteLine(ex.Message)
				Return Nothing
			End Try
		End Function

		Private Sub SetNoCache(ByVal outResponse As OutgoingWebResponseContext)
			'Tell client not to cache
			outResponse.Headers.Add("Pragma", "no-cache")
			outResponse.Headers.Add("Cache-Control", "must-revalidate")
			outResponse.Headers.Add("Cache-Control", "no-cache")
			outResponse.Headers.Add("Cache-Control", "no-store")
			outResponse.Headers.Add("Expires", "0")
		End Sub

		#End Region
	End Class
End Namespace