Imports Microsoft.VisualBasic
Imports System
Imports System.IO
Imports System.Net
Imports System.ServiceModel
Imports System.ServiceModel.Activation
Imports System.ServiceModel.Web
Imports System.Text
Imports System.Web

Namespace SilverTwitWS
	<AspNetCompatibilityRequirements(RequirementsMode := AspNetCompatibilityRequirementsMode.Allowed)> _
	Public Class TwitterService
		Implements ITwitterService
		Private Const tweetSource As String = "SilverTwit"

		Public Function FindTwitterTweets() As String
			Dim wc As New WebClient()
			Dim uri As String = String.Format("http://twitter.com/statuses/public_timeline.xml")
			Dim data As String = wc.DownloadString(uri)
			Return data
		End Function

		Public Function FindTweets(ByVal username As String, ByVal password As String) As String Implements ITwitterService.FindTweets
			Return GetRequest(String.Format("http://twitter.com/statuses/friends_timeline/{0}.xml", username), username, password)
		End Function

		Public Function FindPublicTweets() As String Implements ITwitterService.FindPublicTweets
			Return GetRequest("http://twitter.com/statuses/public_timeline.xml")
		End Function

		Public Function FindArchivedTweets(ByVal username As String, ByVal password As String) As String Implements ITwitterService.FindArchivedTweets
			Return GetRequest(String.Format("http://twitter.com/statuses/user_timeline/{0}.xml", username), username, password)
		End Function

		Public Function FindReplies(ByVal username As String, ByVal password As String) As String Implements ITwitterService.FindReplies
			Return GetRequest("http://twitter.com/statuses/replies.xml", username, password)
		End Function

		Public Function FindDirectMessages(ByVal username As String, ByVal password As String) As String Implements ITwitterService.FindDirectMessages
			Return GetRequest("http://twitter.com/direct_messages.xml", username, password)
		End Function

		Public Function FindFavorites(ByVal username As String, ByVal password As String) As String Implements ITwitterService.FindFavorites
			Return GetRequest("http://twitter.com/favourings.xml", username, password)
		End Function

		Public Function PostStatusUpdate(ByVal message As String, ByVal username As String, ByVal password As String) As String Implements ITwitterService.PostStatusUpdate
			message = message.Trim()
			If message.Length > 140 Then
				message = message.Substring(0, 140)
			End If

			Dim parameters As String = String.Format("status={0}&source={1}", HttpUtility.HtmlEncode(message), tweetSource)

			Return PostRequest("http://twitter.com/statuses/update.xml", username, password, parameters)
		End Function

		Public Function PostCreateTinyUrl(ByVal longUrl As String) As String Implements ITwitterService.PostCreateTinyUrl
			Return PostRequest(String.Format("http://tinyurl.com/api-create.php?url={0}/", longUrl))
		End Function

		Public Function AuthenticateUser(ByVal username As String, ByVal password As String) As Boolean Implements ITwitterService.AuthenticateUser
			Try
				Dim temp As String = GetRequest("http://twitter.com/account/verify_credentials.xml", username, password)
				Return True
			Catch ex As Exception
				Return False
			End Try
		End Function

		#Region "Private Helper Methods"

		Private Function GetRequest(ByVal uri As String) As String
			Return GetRequest(uri, String.Empty, String.Empty)
		End Function

		Private Function GetRequest(ByVal uri As String, ByVal username As String, ByVal password As String) As String
			Dim responseContext As OutgoingWebResponseContext = WebOperationContext.Current.OutgoingResponse
			responseContext.Headers.Add("Cache-Control", "no-cache")
			responseContext.Headers.Add("Pragma", "no-cache")
			responseContext.Headers.Add("Expires", GetFormattedCacheTime(10)) 'TODO: Write the method to set this in seconds

			Dim useWebClient As Boolean = False
			If useWebClient Then
				Dim wc As New WebClient()
				If (Not String.IsNullOrEmpty(username)) AndAlso (Not String.IsNullOrEmpty(password)) Then
					wc.Credentials = New NetworkCredential(username, password)
				End If
				Dim data As String = wc.DownloadString(uri)
				responseContext.StatusCode = HttpStatusCode.OK
				Return data
			Else
				Dim request As WebRequest = HttpWebRequest.Create(uri)

				If (Not String.IsNullOrEmpty(username)) AndAlso (Not String.IsNullOrEmpty(password)) Then
					request.Credentials = New NetworkCredential(username, password)
				End If

				Dim res As WebResponse = request.GetResponse()
				Dim stream As New StreamReader(res.GetResponseStream())
				Dim data As String = stream.ReadToEnd()
				stream.Close()
				responseContext.StatusCode = HttpStatusCode.OK
				Return data
			End If
		End Function

		Private Function GetFormattedCacheTime(ByVal i As Integer) As String
			Dim value As String = DateTime.Now.Add(New TimeSpan(0, 0, 0, i)).ToLongDateString()
			Return value
		End Function

		Private Function PostRequest(ByVal uri As String) As String
			Return PostRequest(uri, String.Empty, String.Empty, String.Empty)
		End Function

		Private Function PostRequest(ByVal uri As String, ByVal username As String, ByVal password As String, ByVal parameters As String) As String
			Dim returnData As String
			Dim webRequest As WebRequest = HttpWebRequest.Create(uri)

			If (Not String.IsNullOrEmpty(username)) AndAlso (Not String.IsNullOrEmpty(password)) Then
				webRequest.Credentials = New NetworkCredential(username, password)
			End If

			webRequest.ContentType = "application/x-www-form-urlencoded"
			webRequest.Method = "POST"
			Dim encoding As New ASCIIEncoding()
			Dim bytes() As Byte = encoding.GetBytes(parameters)

			Dim os As Stream = Nothing
			Try
				webRequest.ContentLength = bytes.Length
				os = webRequest.GetRequestStream()
				os.Write(bytes, 0, bytes.Length)
			Catch ex As WebException
				Throw ex
			Finally
				If os IsNot Nothing Then
					os.Close()
				End If
			End Try

			Try
				Dim webResponse As WebResponse = webRequest.GetResponse()
				If webResponse Is Nothing Then
					Return Nothing
				End If

				Dim sr As New StreamReader(webResponse.GetResponseStream())
				returnData = sr.ReadToEnd().Trim()
				sr.Close()
			Catch ex As WebException
				Throw ex
			End Try

			Return returnData
		End Function

		#End Region
	End Class
End Namespace

